<?php

namespace Drupal\simple_analytics\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\simple_analytics\SimpleAnalyticsHelper;

/**
 * Simple Analytics Configuration form.
 */
class SimpleAnalyticsConfig extends ConfigFormBase {

  /**
   * Get Form ID.
   */
  public function getFormId() {
    return 'simple_analytics_form';
  }

  /**
   * Build Form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = parent::buildForm($form, $form_state);

    $config = SimpleAnalyticsHelper::getConfig();

    $form['google'] = [
      '#type' => 'details',
      '#title' => $this->t("Google Analytics"),
      '#open' => $config->get('google-id') ? TRUE : FALSE,
    ];
    $form['google']['google-id'] = [
      '#type' => 'textfield',
      '#title' => $this->t("Google Analytics ID"),
      '#default_value' => $config->get('google-id'),
    ];

    $form['piwik'] = [
      '#type' => 'details',
      '#title' => $this->t("Piwik Analytics"),
      '#open' => $config->get('piwik-uri') ? TRUE : FALSE,
    ];
    $form['piwik']['piwik-uri'] = [
      '#type' => 'textfield',
      '#description' => $this->t('Example : http://www.example.com/piwik'),
      '#title' => $this->t("Piwik Analytics URL"),
      '#default_value' => $config->get('piwik-uri') ? "http:" . $config->get('piwik-uri') : "",
    ];
    $form['piwik']['piwik-id'] = [
      '#type' => 'textfield',
      '#title' => $this->t("Piwik Analytics ID"),
      '#default_value' => $config->get('piwik-id'),
      '#states' => [
        'invisible' => [
          ':input[id="edit-piwik-uri"]' => ['value' => ''],
        ],
      ],
    ];

    $form['custom'] = [
      '#type' => 'details',
      '#title' => $this->t("Custom"),
      '#open' => $config->get('custom') ? TRUE : FALSE,
    ];
    $form['custom']['custom'] = [
      '#type' => 'textarea',
      '#title' => $this->t("Custom Script"),
      '#description' => $this->t("The code must contains 'script' tag. Ex : @example", ['@example' => '<script type="text/javascript">....']),
      '#default_value' => $config->get('custom'),
    ];
    $form['custom']['custom-noscript'] = [
      '#type' => 'textarea',
      '#title' => $this->t("Custom No Script tracking code"),
      '#description' => $this->t("The code must contains 'noscript' tag. Ex : @example", ['@example' => '<noscript>....']),
      '#default_value' => $config->get('custom-noscript'),
    ];

    $options = [
      7 => '1 Week',
      30 => '1 Month',
      60 => '2 Months',
      180 => '6 Months',
      365 => '1 Year',
    ];

    $form['config'] = [
      '#type' => 'details',
      '#title' => $this->t("Configuration"),
      '#open' => TRUE,
    ];
    $form['config']['displaystat'] = [
      '#type' => 'select',
      '#title' => $this->t('Number of days to display'),
      '#options' => $options,
      '#default_value' => $config->get('displaystat'),
    ];

    $form['config']['track_admin'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Track Admin Pages"),
      '#default_value' => $config->get('track_admin'),
    ];
    $form['config']['track_auth'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Track Authenticated Users"),
      '#default_value' => $config->get('track_auth'),
    ];
    $form['config']['sa_tracker'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Use Simple Analytics internal tracker"),
      '#default_value' => $config->get('sa_tracker'),
    ];
    $form['config']['lib_chartist'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Use Chartist-JS graph"),
      '#default_value' => $config->get('lib_chartist'),
    ];
    if (!simple_analytics_check_requirements()) {
      $form['config']['lib_chartist']['#attributes']['disabled'] = 'disabled';
      $desc = $this->t("To activate, Please add the Chartist-JS library");
      $form['config']['lib_chartist']['#description'] = $desc;
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    if ($form_state->getValue('piwik-uri') && $form_state->getValue('piwik-id') === "") {
      $form_state->setErrorByName('piwik-id', $this->t('If you set a piwik URL, You must also specify the Piwik ID.'));
    }
    elseif (!$form_state->getValue('piwik-uri') && $form_state->getValue('piwik-id') !== "") {
      $form['piwik']['piwik-id']['#value'] = '';
      $form_state->setErrorByName('piwik-uri', $this->t('If you set a piwik ID, You must also specify the Piwik URL (Id removed, you can save now).'));
    }
    if ($form_state->getValue('piwik-uri') && filter_var($form_state->getValue('piwik-uri'), FILTER_VALIDATE_URL) === FALSE) {
      $form_state->setErrorByName('piwik-uri', $this->t('Piwik URL is incorrect.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $piwik_url = $form_state->getValue('piwik-uri');
    $piwik_url = str_replace(["http:", "https:"], "", $piwik_url);
    if (substr($piwik_url, -1) === '/') {
      $piwik_url = substr($piwik_url, 0, -1);
    }

    $config = SimpleAnalyticsHelper::getConfig(TRUE);
    $config->set('google-id', $form_state->getValue('google-id'));
    $config->set('piwik-id', $form_state->getValue('piwik-id'));
    $config->set('custom', $form_state->getValue('custom'));
    $config->set('custom-noscript', $form_state->getValue('custom-noscript'));
    $config->set('track_admin', $form_state->getValue('track_admin'));
    $config->set('track_auth', $form_state->getValue('track_auth'));
    $config->set('displaystat', $form_state->getValue('displaystat'));
    $config->set('lib_chartist', $form_state->getValue('lib_chartist'));
    $config->set('piwik-uri', $piwik_url);
    $config->save();

    return parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      SimpleAnalyticsHelper::getConfigName(),
    ];
  }

}
