<?php

namespace Drupal\simple_analytics\Tests;

use Drupal\simpletest\WebTestBase;

/**
 * Test basic functionality of Simple Analytics module.
 *
 * @group Simple Analytics
 */
class SimpleAnalyticsBasicTest extends WebTestBase {

  /**
   * User with admin Simple Analytics permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  const SA_PIWIK_DOMAIN = "www.domain.com/piwik";
  const SA_PIWIK_ID = 123;
  const SA_GA_ID = "UA-123456-1";

  /**
   * User with admin Simple Analytics permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $userModerator;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'simple_analytics',
    'help',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();

    // Add Simple Analytics view permissions.
    $permissions = [
      'access administration pages',
      'simple_analytics view_history',
      'simple_analytics view_details',
    ];

    // User to set up simple_analytics.
    $this->userModerator = $this->drupalCreateUser($permissions);

    // Add Simple Analytics admin permissions.
    $permissions[] = 'simple_analytics admin';
    $this->admin_user = $this->drupalCreateUser($permissions, 'SimpleAnalyticsAdmin', TRUE);
  }

  /**
   * Tests if configuration is possible.
   */
  public function testSimpleAnalyticsConfiguration() {
    // Login ad admin.
    $this->drupalLogin($this->admin_user);

    // Check if Configure links are available on menus.
    // Requires 'administer modules' permission.
    $this->drupalGet('admin/modules');
    $this->assertRaw('admin/config/system/simple-analytics', '[testSimpleAnalyticsConfiguration]: Configure link from Extend page to Simple Analytics Settings page exists.');
    $this->drupalGet('admin/config/system');
    $this->assertRaw('admin/config/system/simple-analytics', '[testSimpleAnalyticsConfiguration]: Configure link from System Reports to Simple Analytics Settings page exists.');

    // Check for setting page's presence.
    $this->drupalGet('admin/config/system/simple-analytics');
    $this->assertRaw(t('Simple Analytics settings'), '[testSimpleAnalyticsConfiguration]: Settings page displayed.');

    // Check for Piwik account URL/ID validation..
    $edit['piwik-uri'] = 'http://www.domain.com/piwik';
    $this->drupalPostForm('admin/config/system/simple-analytics', $edit, t('Save configuration'));
    $this->assertRaw(t('If you set a piwik URL, You must also specify the Piwik ID.'), '[testSimpleAnalyticsConfiguration]: Invalid Web Property ID number validated.');
  }

  /**
   * Tests if help sections are shown.
   */
  public function testSimpleAnalyticsHelp() {
    // Requires help.module.
    // Login ad admin.
    $this->drupalLogin($this->admin_user);

    // Check help page.
    $this->drupalGet('admin/help/simple_analytics');
    $this->assertText('Simple analytics allow to integrate a site analytics code easily', '[testSimpleAnalyticsHelp]: Simple Analytics help text shown in help section.');
  }

  /**
   * Tests if page visibility works.
   */
  public function testSimpleAnalyticsPageVisibility() {
    // Verify that no GA and Piwik tracking code is embedded into the webpage;
    // only the internal tracking code is present..
    $this->drupalGet('');
    $this->assertRaw('/simple_analytics/track', '[testSimpleAnalyticsAdminPageVisibility]: Simple Tracking code is displayed.');

    $this->drupalGet('');
    $this->assertNoRaw('//www.google-analytics.com/analytics.js', '[testSimpleAnalyticsAdminPageVisibility]: GA Tracking code is not displayed without UA code configured.');
    $this->assertNoRaw("_paq.push(['setTrackerUrl', u+'piwik.php'])", '[testSimpleAnalyticsAdminPageVisibility]: Piwik Tracking code is not displayed without URL and ID configured.');
  }

  /**
   * Tests if page visibility works.
   */
  public function testSimpleAnalyticsAdminPageVisibility() {
    // Login ad admin.
    $this->drupalLogin($this->admin_user);

    // GA and Piwik configuration.
    $config = $this->config('simple_analytics.settings');
    $config->set('google-id', self::SA_GA_ID);
    $config->set('piwik-uri', 'http://' . self::SA_PIWIK_DOMAIN);
    $config->set('piwik-id', self::SA_PIWIK_ID);
    $config->set('track_admin', TRUE);
    $config->set('track_auth', TRUE);
    $config->save();

    // Verify that no GA and Piwik tracking code is embedded into the webpage;
    // Check tracking code visibility on admin pages.
    $this->drupalGet('admin');
    $this->assertRaw('/simple_analytics/track', '[testSimpleAnalyticsAdminPageVisibility]: Simple Tracking code is displayed on admin pages.');
    $this->assertRaw('www.google-analytics.com/analytics.js', '[testSimpleAnalyticsAdminPageVisibility]: Google Tracking code is displayed.');
    $this->assertRaw(self::SA_PIWIK_DOMAIN, '[testSimpleAnalyticsAdminPageVisibility]: Piwik Tracking code is displayed.');

    // Recheck code on home.
    $this->drupalGet('');
    $this->assertRaw('/simple_analytics/track', '[testSimpleAnalyticsAdminPageVisibility]: Simple Tracking code is displayed.');

    // Test whether 404 not found tracking code is shown on non-existent pages.
    $this->drupalGet($this->randomMachineName(64));
    $this->assertResponse(404);
    $this->assertRaw('/simple_analytics/track', '[testSimpleAnalyticsPageVisibility]: Tracking code shown on non-existent page.');
  }

  /**
   * Tests if tracking code is properly added to the page.
   */
  public function testSimpleAnalyticsTrackingCode() {
    // Check internal Tracking page is accessible for everyone and success.
    $this->drupalGet('simple_analytics/track');
    $this->assertResponse(200);
    $this->assertRaw('[{"rasult":"OK"}]', '[testSimpleAnalyticsTrackingCode]: Tracking page accessible and track success.');
  }

}
